<?php

/**
 * @file
 * An implementation of an API for playlist manipulation.  Provides   
 * theming functions and handles general playlist functions. 
 **/



//-------------------------------------------------------------------
//  P l a y l i s t    M a n a g e m e n t
//-------------------------------------------------------------------

/**
 * This function is a menu callback.  You send it the name of your playlist,
 * (i.e. audio playlist, favorite nodes, video playlist, slideshow, photoblog, etc...)
 * The playlist name will be inserted into the 'relationship type' column in the database,
 * thus distinguishing your playlist from another type of playlist. 
 **/
function playlist_manage_files($playlist_name){
  //grab the parameters from the query...
  $parent_id = arg(1); //the nid of the playlist node...
  $action = arg(3); //options are either add or remove
  $child_id = arg(4); //keep in mind here that $child_id is actually an audio node id...

  //is this an 'add' or a remove?
  if ($action == "add") playlist_add_item($parent_id, $child_id);
  if ($action == "remove") playlist_remove_item($parent_id, $child_id);

  //add extra css and javascript for the form...
  //add style sheet...
  //This is pretty brutal to add AJAX stuff like this ------- TODO: use external AJAX modules

  drupal_set_html_head('<style type="text/css">@import url('.drupal_get_path('module','playlist').'/playlist.css);</style>
  <script src="'.drupal_get_path('module','playlist').'/scripts/prototype.js" type="text/javascript"></script>
  <script src="'.drupal_get_path('module','playlist').'/scripts/effects.js" type="text/javascript"></script>
  <script src="'.drupal_get_path('module','playlist').'/scripts/dragdrop.js" type="text/javascript"></script>
  <script src="'.drupal_get_path('module','playlist').'/scripts/control.js" type="text/javascript"></script>');


  //grab a list of all the files that are in this playlist...
  $output .= form_group("Manage " . $playlist_name , _playlist_get_list($parent_id), "You can reorder this list by dragging an item to a new position.");

  //here is the magic AJAX line for sorting a list
  $output .= "<script type=\"text/javascript\">Sortable.create('list', {onUpdate:function(){new Ajax.Updater('list-info', '".url('playlist/ajax/order/'.$parent_id, null,null,true)."', {onComplete:function(request){new Effect.Highlight('list',{});}, parameters:Sortable.serialize('list'), evalScripts:true, asynchronous:true})}})</script>";
  if(function_exists(audio_search)){
  $output .= form_group("Search for items to add to your " .$playlist_name, _playlist_search_output($playlist_name, $parent_id), '');
  }
  print theme('page', $output);
}


/**
 * adds an audio file to this playlist node in the node_relate table...
 */
function playlist_add_item($parent_id, $child_id){
  //do some error checking..
  if (!is_numeric($parent_id) || !is_numeric($child_id)) drupal_not_found();
  //otherwise...

  //make sure this isn't already in the playlist!
  $exist = db_result(db_query("SELECT child_id FROM {playlist_relate} WHERE child_id=%d and parent_id=%d limit 1", $child_id, $parent_id));

  if (!$exist){
	//find highest id
	db_query("INSERT INTO {playlist_relate} (rid, parent_id, child_id, weight) VALUES (%d, %d, '%d','%d')",
		 '', $parent_id, $child_id, _playlist_next_weight_int($parent_id));
	drupal_set_message("The file has been added to the playlist.");
	return 1;
  }
  //drupal_set_message("It appears that this file has already been added to this playlist.", 'error');
}


/**
 * Find the next weight to insert into the table
 */
function _playlist_next_weight_int($parent_id){
  $max = db_result(db_query("SELECT MAX(weight) FROM {playlist_relate} WHERE parent_id=%d limit 1", $parent_id));
  return (is_null($max)) ? 0 : ++$max;
}



/**
 * removes an audio file from this playlist node in the node_relate table...
 */
function playlist_remove_item($parent_id, $child_id){
  //do some error checking..
  if (!is_numeric($parent_id) || !is_numeric($child_id)) drupal_not_found();
  //delete from album
  db_query("DELETE FROM {playlist_relate} WHERE parent_id=%d and child_id=%d", $parent_id, $child_id);
  //set message
  drupal_set_message("The file has been removed from this playlist.");
}



/**
 * removes an audio file from all album nodes in the node_relate table...
 */
function _playlist_remove_all_parents($nid){
  if (!is_numeric($nid)) drupal_not_found();

  //delete from album
  db_query("DELETE FROM {playlist_relate} WHERE child_id=%d", $nid);
}



/**
 * This function basically outputs all the playlist items in a list.
 */
function _playlist_get_list($parent_id){	
  $children_list = array();
  $result = db_query("SELECT child_id FROM {playlist_relate} WHERE parent_id = %d ORDER BY weight ASC", $parent_id);
  while ($row = db_fetch_array($result)) {
   $children_list[] .= $row['child_id'];
  }
  
  $output = '<ul id="list">';
  $result = db_query("SELECT child_id FROM {playlist_relate} WHERE parent_id = %d ORDER BY weight ASC", $parent_id);
  $count = 0;
  while ($row = db_fetch_array($result)) {
	//echo " fid is:" . $row['fid'].":";
	$up_link = l(' [up] ', 'node/'.$parent_id .'/swap/'.$row['child_id'] .'/' . $row['child_id']);
	$down_link = l(' [down] ', 'node/'.$parent_id .'/swap/'.$row['child_id'] .'/' . $row['child_id']);
	if($children_list[$count - 1]) $up_link = l(' [up] ', 'node/'.$parent_id .'/swap/'.$row['child_id'] .'/' . $children_list[$count - 1]);
	if($children_list[$count + 1]) $down_link = l(' [down] ', 'node/'.$parent_id .'/swap/'.$row['child_id'] .'/' . $children_list[$count + 1]);
	$playlist_item_node = node_load(array('nid' => $row['child_id']));
	$output .= '<li id="item_'.$row['child_id'].'">' . _playlist_item($playlist_item_node, $up_link, $down_link) . '</li>';
    $count++;
  }
  $output .= '</ul>';
  $output .= '<p id="list-info"></p>';

  return $output;
}

/**
 * Saves the order of the audio files.
 * Called via AJAX
 */
function playlist_order($parent_id){
  $playlist_items = $_REQUEST['list'];
  foreach ($playlist_items as $child_id) {
  	db_query("UPDATE {playlist_relate} SET weight = '%d' WHERE child_id=%d AND parent_id=%d", $i++, $child_id, $parent_id);
  }
  drupal_set_message("Your changes have been saved.");
  print theme('status_messages');
}

/**
 * Swaps the order of two audio files
 * Called by the 'move up' and 'move down' buttons.
 */
function playlist_swap(){
  $parent_id = arg(1);
  $child_1 = arg(3);
  $child_2 = arg(4);
  
  $weight_1 = db_fetch_object(db_query("SELECT weight FROM {playlist_relate} WHERE child_id=%d AND parent_id=%d", $child_1, $parent_id));
  $weight_2 = db_fetch_object(db_query("SELECT weight FROM {playlist_relate} WHERE child_id=%d AND parent_id=%d", $child_2, $parent_id));
  if($weight_1 && $weight_2){
    db_query("UPDATE {playlist_relate} SET weight = '%d' WHERE child_id=%d AND parent_id=%d", $weight_2->weight, $child_1, $parent_id);
    db_query("UPDATE {playlist_relate} SET weight = '%d' WHERE child_id=%d AND parent_id=%d", $weight_1->weight, $child_2, $parent_id);
    drupal_set_message("Your changes have been saved.");
  }

  drupal_goto('node/'.arg(1).'/manage');
}



/**
 * Display a single playlist item in a box (that is dragged around)...
 */
function _playlist_item($playlist_item_node, $up_link, $down_link){
$type = $playlist_item_node->type;
$text = "";
  //TODO: Ajaxify 'remove function!...
  //TODO: themeable..
  $drag_image = theme_image(drupal_get_path('module', 'playlist') . '/images/drag_me.gif', "Drag item", "Drag item");
  $remove_from_playlist_link = l(theme_image(drupal_get_path('module', 'playlist').'/images/icon_delete.gif',
	"Remove item from playlist", "Remove item from playlist"),
	'node/'. arg(1) .'/manage/remove/'.$playlist_item_node->nid, null, null, null, false, true);
  switch($type) {
	case 'audio':
	  if($playlist_item_node->audio['title']) {
	    $artist = $playlist_item_node->audio['artist'] ? ' by ' . $playlist_item_node->audio['artist'] : "";
	    $text .= $remove_from_playlist_link . '<span class="spacer"></span> ' . $drag_image . $up_link . $down_link .'<span class="spacer"></span>' . theme('mp3_player',$playlist_item_node) . ' ' . l($playlist_item_node->audio['title'] . $artist, 'node/'.$playlist_item_node->nid); 
	  }
	  else {
	    $text .= $remove_from_playlist_link . '<span class="spacer"></span> ' . $drag_image . $up_link . $down_link . '<span class="spacer"></span>' . theme('mp3_player',$playlist_item_node) . ' ' . l($playlist_item_node->title,'node/'.$playlist_item_node->nid);;  
	  }
	break;
	default:
        $text .= $remove_from_playlist_link . '<span class="spacer"></span> ' . $drag_image . $up_link . $down_link . '<span class="spacer"></span>' . l($playlist_item_node->title,'node/'.$playlist_item_node->nid);
    break;
  }
  return $text;
}


/**
 * METHOD #1: ADDING A PLAYLIST ITEM TO A PLAYLIST FROM THE PLAYLIST EDIT VIEW
 * The drop down menu that is displayed on audio pages to let a user add a
 * new track to an album.
 */
function  playlist_select_form($node){
  global $user;
  
  if(user_access('administer playlists')) {
    $result = db_query("SELECT title, nid FROM {node} WHERE type='%s' ORDER BY title ASC", 'playlist');
  }
  else {
    $result = db_query("SELECT title, nid FROM {node} n WHERE type='%s' AND uid = %d ORDER BY title ASC", 'playlist', $user->uid);
  }
  
  //form_select($title, $name, $value, $options, $description = NULL, $extra = 0, $multiple = FALSE, $required = FALSE)
  $items = array('-1' => '<none>');

  while($item = db_fetch_object($result)){
	$items[$item->nid] .= $item->title;
  }

  //grab the selected albums...
  $result = db_query("SELECT parent_id FROM {playlist_relate} WHERE child_id = %d", $node->nid); //'child_id' is the node id of the audio node...
  $selected = array();
  while($item = db_fetch_object($result)) {
	$selected[] .= $item->parent_id;
  }
  $selected = (empty($selected)) ? '-1' : $selected;

  $dropdown .= form_select(t('Add to playlist'), "playlist_select", $selected, $items,
  "You can add this item to one of these Playlists. You can always change this later.", 0, TRUE, FALSE);

  //grab a list of all the audio files that are in this album...
  $output .= form_group("Playlists", $dropdown);

  return $output;
}


/**
 * METHOD #2: ADD AN ITEM TO PLAYLIST(S) FROM THE ITEM VIEW
 * Implementation of function playlist_additem()
 * Handles the intermediary form used to add an item to a playlist.
 *
 */
function playlist_choose() {
  global $user;
  $uid = $user->uid;
  $child_id = arg(2);
  //_print_rp($_POST);

  if($_POST['edit'] and ($_POST['op'] == "update")){
	foreach ($_POST['edit'] as $parent_id => $value) {
	  if ($value) {
		playlist_add_item($parent_id, $child_id);
	  } else {
		playlist_remove_item($parent_id, $child_id);
	  }
	}
	drupal_goto($_REQUEST["REFERRER"]);
  }


  $output = '<ul><li>'. l(t('Create a new playlist.'), "node/add/playlist") .'</li></ul>';
  // get the playlist IDs where this node is already added
  $result = db_query("SELECT parent_id FROM {playlist_relate} WHERE child_id = %d", $child_id);
  while($playlist = db_fetch_object($result)) {
	//_print_rp($playlist);
	$selected_array[] = $playlist->parent_id;
  }
  //display a list of user's playlists with checkboxes
  $result = db_query("SELECT title, nid, uid FROM {node} WHERE type='%s' AND uid = %d ORDER BY title ASC", 'playlist', $uid);
  while($playlist = db_fetch_object($result)) {
    $title = $playlist->title;
    $parent_id =  $playlist->nid;
    $checked = FALSE;
    // check to see if the item is already in a given playlist
    if($selected_array) {
	  foreach($selected_array as $value) {
		if($value == $parent_id) {
		  $checked = TRUE;
		}
	  }
	}
	$form .= form_checkbox($title, $parent_id, $value = 1, $checked, $description = NULL, $attributes = NULL, $required = FALSE);
  }

  //output everything to the browser
  $form .= form_submit(t('update'));
  $output .= form($form, 'post', NULL, NULL);
  print theme('page', $output);
}

/**
 * METHOD #3: ITEM ADDED TO PLAYLIST USING SEARCH-LIKE CAPABILITIES FROM THE "MANAGE" TAB
 * Renders the audio table on the media page
 */
function _playlist_search_output($playlist_name, $playlist_nid) {
switch($playlist_name){
 case 'audio playlist':
 $output .= playlist_search_view('audio', $playlist_nid); 
 //$output .= search_form(url('node/'.$playlist_nid.'/manage'), null, 'audio', $prompt = 'Search for items to add to your' .$playlist_name);
 break;
 default:
}


return $output;

}


/**
 * This is a copy of the search_view menu callback, slightly modified. Get rid of this soon. 
 * Maybe just replace this with a custom search function?
 *
 */
function playlist_search_view($type, $playlist_nid) {

  $keys = $_POST['edit']['keys'];

  if (user_access('search content')) {
    // Only perform search if there is non-whitespace search term:
    if (trim($keys)) {
      // Log the search keys:

      // Collect the search results:
      $results = search_data($keys, $type);

      if ($results) {
        $results = theme('box', t('Search results'), $results);
      }
      else {
        $results = theme('box', t('Your search yielded no results'), search_help('search#noresults'));
      }
    }
    else if (isset($_POST['edit'])) {
      form_set_error('keys', t('Please enter some keywords.'));
    }

    // Construct the search form.
    // Note, we do this last because of the form_set_error() above.
    $output = search_form(url('node/'.$playlist_nid.'/manage'), null, $type, $prompt = null);

    $output .= $results;

    return $output;
  }
  else {
    drupal_access_denied();
  }
} 







 
?>